@@ -48,18 +48,11 @@ sub emit_perl5 {
         $self->{term}->set_captures_to_array;
         return
             '(do { '
-            .   'my $last_match_null = 0; '
             .   'my $m = $MATCH; '
             .   'my $to = $MATCH->{to}; '
             .   'my $count = 0; '
-            .   'while (' . $self->{term}->emit_perl5() . ' && ($last_match_null < 2)) '
+            .   'while (' . $self->{term}->emit_perl5() . ') '
             .   '{ '
-            .       'if ($to == $MATCH->{to}) { '
-            .           '$last_match_null = $last_match_null + 1; '
-            .       '} '
-            .       'else { '
-            .           '$last_match_null = 0; '
-            .       '}; '
             .       '$m = $MATCH; '
             .       '$to = $MATCH->{to}; '
             .       '$count = $count + 1; '
@@ -73,17 +66,10 @@ sub emit_perl5 {
         $self->{term}->set_captures_to_array;
         return
             '(do { '
-            .   'my $last_match_null = 0; '
             .   'my $m = $MATCH; '
             .   'my $to = $MATCH->{to}; '
-            .   'while (' . $self->{term}->emit_perl5() . ' && ($last_match_null < 2)) '
+            .   'while (' . $self->{term}->emit_perl5() . ') '
             .   '{ '
-            .       'if ($to == $MATCH->{to}) { '
-            .           '$last_match_null = $last_match_null + 1; '
-            .       '} '
-            .       'else { '
-            .           '$last_match_null = 0; '
-            .       '}; '
             .       '$m = $MATCH; '
             .       '$to = $MATCH->{to}; '
             .   '}; '
@@ -180,12 +166,7 @@ sub emit_perl5 {
     elsif ($self->{captures} > 1) {
         $code = 'if ($m2) { '
                 .   '$MATCH->{to} = $m2->{to}; '
-                .   'if (exists $MATCH->{\'' . $self->{metasyntax} . '\'}) { '
-                .       'push @{ $MATCH->{\'' . $self->{metasyntax} . '\'} }, $m2; '
-                .   '} '
-                .   'else { '
-                .       '$MATCH->{\'' . $self->{metasyntax} . '\'} = [ $m2 ]; '
-                .   '}; '
+                .   'push @{ $MATCH->{\'' . $self->{metasyntax} . '\'} }, $m2; '
                 .   '1 '
                 . '} else { 0 }; '
     }
