@@ -1544,7 +1544,7 @@ package Perlito5::AST::Apply;
                     namespace => '',
                     arguments => [
                        Perlito5::AST::Apply->new(
-                          code => 'do_file',
+                          code => 'slurp_file',
                           namespace => 'Perlito5::Grammar::Use',
                           arguments => $self->{arguments}
                         )
@@ -1845,9 +1845,7 @@ package Perlito5::AST::Apply;
             else {
                 # this subroutine was never declared
                 if ($self->{bareword}) {
-                    if ( $Perlito5::STRICT ) {
-                        die 'Bareword "' . $name . '" not allowed while "strict subs" in use';
-                    }
+                    # die 'Bareword "' . $name . '" not allowed while "strict subs" in use';
                     return Perlito5::JavaScript3::escape_string( 
                             ($self->{namespace} ? $self->{namespace} . '::' : "") . $name 
                         );
@@ -2147,7 +2145,7 @@ This module generates JavaScript3 code for the Perlito compiler.
 =head1 AUTHORS
 
 Flavio Soibelmann Glock <fglock@gmail.com>.
-The Pugs Team E<lt>perl6-compiler@perl.orgE<gt>.
+The Pugs Team.
 
 =head1 COPYRIGHT
 
